@@ -1,6 +1,6 @@
 <?php
-// PHP Weathermap 0.96a
-// Copyright Howard Jones, 2005-2009 howie@thingy.com
+// PHP Weathermap 0.97a
+// Copyright Howard Jones, 2005-2010 howie@thingy.com
 // http://www.network-weathermap.com/
 // Released under the GNU Public License
 
@@ -9,6 +9,7 @@ require_once "HTML_ImageMap.class.php";
 class WeatherMapLink extends WeatherMapItem
 {
 	var $owner,                $name;
+	var $id;
 	var $maphtml;
 	var $a,                    $b; // the ends - references to nodes
 	var $width,                $arrowstyle, $linkstyle;
@@ -32,6 +33,7 @@ class WeatherMapLink extends WeatherMapItem
 	var $vialist = array();
 	var $viastyle;
 	var $usescale, $duplex; 
+	var $scaletype;
 	var $outlinecolour;
 	var $bwoutlinecolour;
 	var $bwboxcolour;
@@ -68,6 +70,7 @@ class WeatherMapLink extends WeatherMapItem
 			'arrowstyle' => 'classic',
 			'viastyle' => 'curved',
 			'usescale' => 'DEFAULT',
+			'scaletype' => 'percent',
 			'targets' => array(),
 			'duplex' => 'full',
 			'infourl' => array('',''),
@@ -132,6 +135,7 @@ class WeatherMapLink extends WeatherMapItem
 		// to stop the editor tanking, now that colours are decided earlier in ReadData
 		$this->colours[IN] = new Colour(192,192,192);
 		$this->colours[OUT] = new Colour(192,192,192);
+		$this->id = $newowner->next_id++;
 	}
 
 	function my_type() {  return "LINK"; }
@@ -187,8 +191,6 @@ class WeatherMapLink extends WeatherMapItem
 			
 				list($textlength, $textheight) = $this->owner->myimagestringsize($this->commentfont, $comment);
 				
-				// XXX - redundant extra variable
-				// $startindex = $start[$dir];
 				$extra_percent = $commentpos[$dir];
 				
 				// $font = $this->commentfont;
@@ -294,16 +296,11 @@ class WeatherMapLink extends WeatherMapItem
 		$x2=$map->nodes[$this->b->name]->x;
 		$y2=$map->nodes[$this->b->name]->y;
 		
-		if(is_null($x1)) { warn("LINK ".$this->name." uses a NODE with no POSITION!\n"); return; }
-		if(is_null($y1)) { warn("LINK ".$this->name." uses a NODE with no POSITION!\n"); return; }
-		if(is_null($x2)) { warn("LINK ".$this->name." uses a NODE with no POSITION!\n"); return; }
-		if(is_null($y2)) { warn("LINK ".$this->name." uses a NODE with no POSITION!\n"); return; }
+		if(is_null($x1)) { warn("LINK ".$this->name." uses a NODE with no POSITION! [WMWARN35]\n"); return; }
+		if(is_null($y1)) { warn("LINK ".$this->name." uses a NODE with no POSITION! [WMWARN35]\n"); return; }
+		if(is_null($x2)) { warn("LINK ".$this->name." uses a NODE with no POSITION! [WMWARN35]\n"); return; }
+		if(is_null($y2)) { warn("LINK ".$this->name." uses a NODE with no POSITION! [WMWARN35]\n"); return; }
 		
-		if( ($x1==$x2) && ($y1==$y2) && sizeof($this->vialist)==0)
-		{
-			warn("Zero-length link ".$this->name." skipped.");
-			return;
-		}
 		
 		if( ($this->linkstyle=='twoway') && ($this->labeloffset_in < $this->labeloffset_out) && (intval($map->get_hint("nowarn_bwlabelpos"))==0) )
 		{
@@ -318,6 +315,12 @@ class WeatherMapLink extends WeatherMapItem
 		$x2+=$dx;
 		$y2+=$dy;
 
+		if( ($x1==$x2) && ($y1==$y2) && sizeof($this->vialist)==0)
+		{
+			warn("Zero-length link ".$this->name." skipped. [WMWARN45]");
+			return;
+		}
+
 		$outlinecol = new Colour($this->outlinecolour);
 		$commentcol = new Colour($this->commentfontcolour);
 		
@@ -398,7 +401,7 @@ class WeatherMapLink extends WeatherMapItem
 			// then draw the "curve" itself
 			draw_straight($im, $this->curvepoints,
 				array($link_in_width,$link_out_width), $outline_colour, array($gd_in_colour, $gd_out_colour),
-				$this->name, $map, 50, ($this->linkstyle=='oneway'?TRUE:FALSE) );
+				$this->name, $map, $this->splitpos, ($this->linkstyle=='oneway'?TRUE:FALSE) );
 		}
 
 		if ( !$commentcol->is_none() )
@@ -509,10 +512,11 @@ class WeatherMapLink extends WeatherMapItem
 	function WriteConfig()
 	{
 		$output='';
+		# $output .= "# ID ".$this->id." - first seen in ".$this->defined_in."\n";
 
 		if($this->config_override != '')
 		{
-			$output  = $this->config_override."\n";
+			$output = $this->config_override."\n";
 		}
 		else
 		{
@@ -555,54 +559,62 @@ class WeatherMapLink extends WeatherMapItem
 			{
 				$field = $param[0];
 				$keyword = $param[1];
-							
+						
+				# $output .= "# For $keyword: ".$this->$field." vs ".$dd->$field."\n";
 				if ($this->$field != $dd->$field) 
+				#if (1==1)
 				{ 
 					if($param[2] == CONFIG_TYPE_COLOR) $output.="\t$keyword " . render_colour($this->$field) . "\n"; 
 					if($param[2] == CONFIG_TYPE_LITERAL) $output.="\t$keyword " . $this->$field . "\n"; 
 				}
 			}		
 		
-			if($this->infourl[IN]==$this->infourl[OUT])
+			if ($this->infourl[IN] == $this->infourl[OUT]) {
 				$dirs = array(IN=>""); // only use the IN value, since they're both the same, but don't prefix the output keyword
-			else
+			} else {
 				$dirs = array( IN=>"IN", OUT=>"OUT" );// the full monty two-keyword version
+			}
 						
-			foreach ($dirs as $dir=>$tdir)
-			{
-				if ($this->infourl[$dir] != $dd->infourl[$dir]) { $output.="\t".$tdir."INFOURL " . $this->infourl[$dir] . "\n"; }
+			foreach ($dirs as $dir=>$tdir) {
+				if ($this->infourl[$dir] != $dd->infourl[$dir]) { 
+					$output .= "\t" . $tdir . "INFOURL " . $this->infourl[$dir] . "\n"; 
+				}
 			}
 			
-			if($this->overlibcaption[IN]==$this->overlibcaption[OUT])
+			if ($this->overlibcaption[IN] == $this->overlibcaption[OUT]) {
 				$dirs = array(IN=>""); // only use the IN value, since they're both the same, but don't prefix the output keyword
-			else
+			} else {
 				$dirs = array( IN=>"IN", OUT=>"OUT" );// the full monty two-keyword version
+			}
 						
-			foreach ($dirs as $dir=>$tdir)
-			{
-				if ($this->overlibcaption[$dir] != $dd->overlibcaption[$dir]) { $output.="\t".$tdir."OVERLIBCAPTION " . $this->overlibcaption[$dir] . "\n"; }
+			foreach ($dirs as $dir=>$tdir) {
+				if ($this->overlibcaption[$dir] != $dd->overlibcaption[$dir]) { 
+					$output .= "\t".$tdir."OVERLIBCAPTION " . $this->overlibcaption[$dir] . "\n"; 
+				}
+			}
+		
+			if ($this->notestext[IN] == $this->notestext[OUT]) {
+				$dirs = array(IN=>""); // only use the IN value, since they're both the same, but don't prefix the output keyword
+			} else { 
+				$dirs = array( IN=>"IN", OUT=>"OUT" );// the full monty two-keyword version
 			}
 	
-	
-			if($this->notestext[IN]==$this->notestext[OUT])
-				$dirs = array(IN=>""); // only use the IN value, since they're both the same, but don't prefix the output keyword
-			else
-				$dirs = array( IN=>"IN", OUT=>"OUT" );// the full monty two-keyword version
-	
-			foreach ($dirs as $dir=>$tdir)
-			{
-				if ($this->notestext[$dir] != $dd->notestext[$dir]) { $output.="\t".$tdir."NOTES " . $this->notestext[$dir] . "\n"; }
+			foreach ($dirs as $dir=>$tdir) {
+				if ($this->notestext[$dir] != $dd->notestext[$dir]) {
+					$output .= "\t" . $tdir . "NOTES " . $this->notestext[$dir] . "\n"; 
+				}
 			}
-	
-			
-			if($this->overliburl[IN]==$this->overliburl[OUT])
+				
+			if ($this->overliburl[IN]==$this->overliburl[OUT]) {
 				$dirs = array(IN=>""); // only use the IN value, since they're both the same, but don't prefix the output keyword
-			else
+			} else {
 				$dirs = array( IN=>"IN", OUT=>"OUT" );// the full monty two-keyword version
+			}
 			
-			foreach ($dirs as $dir=>$tdir)
-			{
-				if ($this->overliburl[$dir] != $dd->overliburl[$dir]) { $output.="\t".$tdir."OVERLIBGRAPH " . join(" ",$this->overliburl[$dir]) . "\n"; }
+			foreach ($dirs as $dir=>$tdir) {
+				if ($this->overliburl[$dir] != $dd->overliburl[$dir]) { 
+					$output.="\t".$tdir."OVERLIBGRAPH " . join(" ",$this->overliburl[$dir]) . "\n"; 
+				}
 			}	
 			
 			// if formats have been set, but they're just the longform of the built-in styles, set them back to the built-in styles
@@ -621,7 +633,9 @@ class WeatherMapLink extends WeatherMapItem
 
 			// if specific formats have been set, then the style will be '--'
 			// if it isn't then use the named style
-			if ( ($this->labelstyle != $dd->labelstyle) && ($this->labelstyle != '--') ) { $output.="\tBWLABEL " . $this->labelstyle . "\n"; }
+			if ( ($this->labelstyle != $dd->labelstyle) && ($this->labelstyle != '--') ) { 
+				$output .= "\tBWLABEL " . $this->labelstyle . "\n"; 
+			}
 						
 			// if either IN or OUT field changes, then both must be written because a regular BWLABEL can't do it
 			// XXX this looks wrong
@@ -630,10 +644,9 @@ class WeatherMapLink extends WeatherMapItem
 						
 			if ( ( $this->labelstyle == '--') && ( ($this->bwlabelformats[IN] != $comparison) || ($this->bwlabelformats[OUT]!= '--')) )
 			{
-				$output.="\tINBWFORMAT " . $this->bwlabelformats[IN]. "\n";
-				$output.="\tOUTBWFORMAT " . $this->bwlabelformats[OUT]. "\n";
+				$output .= "\tINBWFORMAT " . $this->bwlabelformats[IN]. "\n";
+				$output .= "\tOUTBWFORMAT " . $this->bwlabelformats[OUT]. "\n";
 			}
-		
 	
 			$comparison = $dd->labeloffset_in;
 			$comparison2 = $dd->labeloffset_out;
@@ -648,63 +661,55 @@ class WeatherMapLink extends WeatherMapItem
 	
 			$comparison=$dd->targets;
 	
-			if ($this->targets != $comparison)
-			{
+			if ($this->targets != $comparison) {
 				$output.="\tTARGET";
 	
-				foreach ($this->targets as $target) 
-				{ 
-					if(strpos($target[4]," ") == FALSE) 
-					{
-						$output.=" " . $target[4]; 
+				foreach ($this->targets as $target) { 
+					if(strpos($target[4]," ") == FALSE) {
+						$output .= " " . $target[4]; 
+					} else {
+						$output .= ' "' . $target[4] . '"'; 
 					}
-					else
-					{
-						$output.=' "' . $target[4].'"'; 
-					}
-				}
-	
-				$output.="\n";
+				}	
+				$output .= "\n";
 			}
-	
-			
-	
-			foreach (array(IN,OUT) as $dir)
-			{
-				if($dir==IN) $tdir="IN";
-				if($dir==OUT) $tdir="OUT";
+				
+			foreach (array(IN,OUT) as $dir) {
+				if ($dir==IN) { 
+					$tdir="IN";
+				}
+				if ($dir==OUT) { 
+					$tdir="OUT";
+				}
 				
 				$comparison=$dd->comments[$dir];
-				if ($this->comments[$dir] != $comparison) { $output.="\t".$tdir."COMMENT " . $this->comments[$dir] . "\n"; }
+				if ($this->comments[$dir] != $comparison) { 
+					$output .= "\t" . $tdir . "COMMENT " . $this->comments[$dir] . "\n"; 
+				}
 			}
-				
+					
+			if (isset($this->a) && isset($this->b))	{
+				$output .= "\tNODES " . $this->a->name;
 	
-			if (isset($this->a) && isset($this->b))
-			{
-				$output.="\tNODES " . $this->a->name;
+				if ($this->a_offset != 'C') {
+					$output .= ":" . $this->a_offset;
+				}
 	
-				if ($this->a_offset != 'C')
-					$output.=":" . $this->a_offset;
+				$output .= " " . $this->b->name;
 	
-				$output.=" " . $this->b->name;
+				if ($this->b_offset != 'C') {
+					$output .= ":" . $this->b_offset;
+				}
 	
-				if ($this->b_offset != 'C')
-					$output.=":" . $this->b_offset;
-	
-				$output.="\n";
+				$output .= "\n";
 			}
 	
-			if (count($this->vialist) > 0)
-			{
-				foreach ($this->vialist as $via)
-				{
-					if(isset($via[2]))
-					{
-						$output.=sprintf("\tVIA %s %d %d\n", $via[2],$via[0], $via[1]);
-					}
-					else
-					{
-						$output.=sprintf("\tVIA %d %d\n", $via[0], $via[1]);
+			if (count($this->vialist) > 0) {
+				foreach ($this->vialist as $via) {
+					if( isset($via[2])) {
+						$output .= sprintf("\tVIA %s %d %d\n", $via[2],$via[0], $via[1]);
+					} else {
+						$output .= sprintf("\tVIA %d %d\n", $via[0], $via[1]);
 					}
 				}
 			}
@@ -737,8 +742,7 @@ class WeatherMapLink extends WeatherMapItem
 			      }
 			}
 	
-			if ($output != '')
-			{
+			if ($output != '') {
 				$output = "LINK " . $this->name . "\n".$output."\n";
 			}
 		}
@@ -749,6 +753,7 @@ class WeatherMapLink extends WeatherMapItem
 	{
 		$js='';
 		$js.="Links[" . js_escape($this->name) . "] = {";
+		$js .= "\"id\":" . $this->id. ", ";
 
 		if (isset($this->a))
 		{
@@ -761,7 +766,13 @@ class WeatherMapLink extends WeatherMapItem
 
 		$tgt='';
 
-		foreach ($this->targets as $target) { $tgt.=$target[4] . ' '; }
+		foreach ($this->targets as $target) { 
+			if(strpos($target[4]," ") == FALSE) {
+				$tgt .= $target[4] . ' ';
+			} else {
+				$tgt .= '"'.$target[4] . '" ';
+			}
+		}
 
 		$js.=js_escape(trim($tgt));
 		$js.=",";
@@ -784,14 +795,15 @@ class WeatherMapLink extends WeatherMapItem
 		$js.="overliburl:" . js_escape(join(" ",$this->overliburl[IN]));
 		
 		$js.="};\n";
+		$js .= "LinkIDs[\"L" . $this->id . "\"] = ". js_escape($this->name) . ";\n";
 		return $js;
 	}
 
 	function asJSON($complete=TRUE)
 	{
-		$js='';
-		$js.="" . js_escape($this->name) . ": {";
-
+		$js = '';
+		$js .= "" . js_escape($this->name) . ": {";
+		$js .= "\"id\":" . $this->id. ", ";
 		if (isset($this->a)) 
 		{
 			$js.="\"a\":\"" . $this->a->name . "\", ";
