@@ -1,6 +1,6 @@
 <?php
-// PHP Weathermap 0.96a
-// Copyright Howard Jones, 2005-2009 howie@thingy.com
+// PHP Weathermap 0.97a
+// Copyright Howard Jones, 2005-2010 howie@thingy.com
 // http://www.network-weathermap.com/
 // Released under the GNU Public License
 
@@ -9,6 +9,7 @@ require_once "HTML_ImageMap.class.php";
 class WeatherMapNode extends WeatherMapItem
 {
 	var $owner;
+	var $id;
 	var $x,	$y;
 	var $original_x, $original_y,$relative_resolved;
 	var $width, $height;
@@ -40,6 +41,7 @@ class WeatherMapNode extends WeatherMapItem
 	var $cachefile;
 	var $usescale;
 	var $useiconscale;
+	var $scaletype, $iconscaletype;
 	var $inscalekey,$outscalekey;
 	var $inscaletag, $outscaletag;
 	# var $incolour,$outcolour;
@@ -62,6 +64,8 @@ class WeatherMapNode extends WeatherMapItem
 				'label' => '',
 				'proclabel' => '',
 				'usescale' => 'DEFAULT',
+				'scaletype' => 'percent',
+				'iconscaletype' => 'percent',
 				'useiconscale' => 'none',
 				'scalevar' => 'in',
 				'template' => ':: DEFAULT ::',
@@ -172,19 +176,32 @@ class WeatherMapNode extends WeatherMapItem
 		{
 			debug("Colorising the icon\n");
 			$pc = 0;
+			$val = 0;
 
 			if($this->iconscalevar == 'in')
 			{
 				$pc = $this->inpercent;
 				$col = $this->colours[IN];
+				$val = $this->bandwidth_in;
 			}
 			if($this->iconscalevar == 'out')
 			{
 				$pc = $this->outpercent;
 				$col = $this->colours[OUT];
+				$val = $this->bandwidth_out;
 			}
 
-			list($colicon,$node_iconscalekey,$icontag) = $map->NewColourFromPercent($pc, $this->useiconscale,$this->name);
+			if($this->iconscaletype=='percent')
+			{
+				list($colicon,$node_iconscalekey,$icontag) = 
+					$map->NewColourFromPercent($pc, $this->useiconscale,$this->name );	
+			}
+			else
+			{
+				// use the absolute value if we aren't doing percentage scales.
+				list($colicon,$node_iconscalekey,$icontag) = 
+					$map->NewColourFromPercent($val, $this->useiconscale,$this->name, FALSE );
+			}
 		}
 
 		// figure out a bounding rectangle for the label
@@ -267,8 +284,6 @@ class WeatherMapNode extends WeatherMapItem
 				debug("Artificial Icon type " .$this->iconfile. " for $this->name\n");
 				// this is an artificial icon - we don't load a file for it
 
-				// XXX - add the actual DRAWING CODE!
-
 				$icon_im = imagecreatetruecolor($this->iconscalew,$this->iconscaleh);
 				imageSaveAlpha($icon_im, TRUE);
 
@@ -424,22 +439,17 @@ class WeatherMapNode extends WeatherMapItem
 
 					$icon_im = imagecreatefromfile($this->iconfile);
 					# $icon_im = imagecreatefrompng($this->iconfile);
-					if(function_exists("imagefilter"))
-					{
-						// warn("Colorising?\n");
-						if (isset($colicon))
-						{
-						//	warn("YES\n");
-							imagefilter($icon_im, IMG_FILTER_COLORIZE, $colicon->r, $colicon->g, $colicon->b);
-						}
-						else
-						{
-						//	warn("NO\n");
-						}
+					if(function_exists("imagefilter") && isset($colicon) && $this->get_hint("use_imagefilter")==1)
+					{						
+						imagefilter($icon_im, IMG_FILTER_COLORIZE, $colicon->r, $colicon->g, $colicon->b);						
 					}
 					else
 					{
-						debug("Skipping unavailable imagefilter() call.\n");
+                                            if(isset($colicon))
+                                            {
+                                                // debug("Skipping unavailable imagefilter() call.\n");
+                                                imagecolorize($icon_im, $colicon->r, $colicon->g, $colicon->b);
+                                            }
 					}
 
 					debug("If this is the last thing in your logs, you probably have a buggy GD library. Get > 2.0.33 or use PHP builtin.\n");
@@ -471,13 +481,13 @@ class WeatherMapNode extends WeatherMapItem
 
 						}
 					}
-					else { warn ("Couldn't open PNG ICON: '" . $this->iconfile . "' - is it a PNG, JPEG or GIF?\n"); }
+					else { warn ("Couldn't open ICON: '" . $this->iconfile . "' - is it a PNG, JPEG or GIF? [WMWARN37]\n"); }
 				}
 				else
 				{
 					if($this->iconfile != 'none')
 					{
-						warn ("ICON '" . $this->iconfile . "' does not exist, or is not readable. Check path and permissions.\n");
+						warn ("ICON '" . $this->iconfile . "' does not exist, or is not readable. Check path and permissions. [WMARN38]\n");
 					}
 				}
 			}
@@ -629,7 +639,7 @@ class WeatherMapNode extends WeatherMapItem
 				}
 				else
 				{
-					warn("You can't make a contrast with 'none'.\n");
+					warn("You can't make a contrast with 'none'. [WMWARN43]\n");
 					$txcol = new Colour(0,0,0);
 				}
 			}
@@ -712,6 +722,8 @@ class WeatherMapNode extends WeatherMapItem
 		// to stop the editor tanking, now that colours are decided earlier in ReadData
 		$this->colours[IN] = new Colour(192,192,192);
 		$this->colours[OUT] = new Colour(192,192,192);
+		
+		$this->id = $newowner->next_id++;
 	}
 
 	function CopyFrom(&$source)
@@ -727,6 +739,8 @@ class WeatherMapNode extends WeatherMapItem
 	function WriteConfig()
 	{
 		$output='';
+				
+		# $output .= "# ID ".$this->id." - first seen in ".$this->defined_in."\n";
 
 		// This allows the editor to wholesale-replace a single node's configuration
 		// at write-time - it should include the leading NODE xyz line (to allow for renaming)
@@ -914,10 +928,11 @@ class WeatherMapNode extends WeatherMapItem
 
 	function asJS()
 	{
-		$js='';
-		$js.="Nodes[" . js_escape($this->name) . "] = {";
-		$js.="x:" . (is_null($this->x)? "'null'" : $this->x) . ", ";
-		$js.="y:" . (is_null($this->y)? "'null'" : $this->y) . ", ";
+		$js = '';
+		$js .= "Nodes[" . js_escape($this->name) . "] = {";
+		$js .= "x:" . (is_null($this->x)? "'null'" : $this->x) . ", ";
+		$js .= "y:" . (is_null($this->y)? "'null'" : $this->y) . ", ";
+		$js .= "\"id\":" . $this->id. ", ";
 		// $js.="y:" . $this->y . ", ";
 		$js.="ox:" . $this->original_x . ", ";
 		$js.="oy:" . $this->original_y . ", ";
@@ -938,7 +953,8 @@ class WeatherMapNode extends WeatherMapItem
 			$js.="iconfile:" . js_escape($this->iconfile);
 		}
 		
-		$js.="};\n";
+		$js .= "};\n";
+		$js .= "NodeIDs[\"N" . $this->id . "\"] = ". js_escape($this->name) . ";\n";
 		return $js;
 	}
 
@@ -946,6 +962,7 @@ class WeatherMapNode extends WeatherMapItem
 	{
 		$js = '';
 		$js .= "" . js_escape($this->name) . ": {";
+		$js .= "\"id\":" . $this->id. ", ";
 		$js .= "\"x\":" . ($this->x - $this->centre_x). ", ";
 		$js .= "\"y\":" . ($this->y - $this->centre_y) . ", ";
 		$js .= "\"cx\":" . $this->centre_x. ", ";
