@@ -4,12 +4,12 @@ require_once 'editor.inc.php';
 require_once 'Weathermap.class.php';
 
 // so that you can't have the editor active, and not know about it.
-$ENABLED=true;
+$ENABLED=false;
 
 if(! $ENABLED)
 {
     print "<p>The editor has not been enabled yet. You need to set ENABLED=true at the top of editor.php</p>";
-    print "<p>Before you do that, you should consider using FilesMatch (in Apache) or similar to limit who can access the editor.</p>";
+    print "<p>Before you do that, you should consider using FilesMatch (in Apache) or similar to limit who can access the editor. There is more information in the install guide section of the manual.</p>";
     exit();
 }
 
@@ -112,6 +112,10 @@ else
 	debug("==========================================================================================================\n");
 	debug("Starting Edit Run: action is $action on $mapname\n");
 	debug("==========================================================================================================\n");
+	
+	# editor_log("\n\n-----------------------------------------------------------------------------\nNEW REQUEST:\n\n");
+	# editor_log(var_log($_REQUEST));
+	
 	$map = new WeatherMap;
 	$map->context = 'editor';
 	
@@ -175,6 +179,7 @@ else
 
 		exit();        
 		break;
+
 	case 'draw':
 		header('Content-type: image/png');
 		$map->ReadConfig($mapfile);
@@ -214,31 +219,29 @@ else
 		exit();
 		break;
         
-        case 'fetch_config':
-                $map->ReadConfig($mapfile);
+    case 'fetch_config':
+        $map->ReadConfig($mapfile);
 		header('Content-type: text/plain');
 		$item_name = $_REQUEST['item_name'];
-                $item_type = $_REQUEST['item_type'];
-                $ok=FALSE;
+		$item_type = $_REQUEST['item_type'];
+		$ok=FALSE;
 
-                if($item_type == 'node')
-                {
-                        if(isset($map->nodes[$item_name]))
-                        {
-                                print $map->nodes[$item_name]->WriteConfig();
-                                $ok=TRUE;
-                        }
-                }
-                if($item_type == 'link')
-                {
-                        if(isset($map->links[$item_name]))
-                        {
-                                print $map->links[$item_name]->WriteConfig();
-                                $ok=TRUE;
-                        }
-                }
-                
-                if(! $ok) { print "# the request item didn't exist. That's probably a bug.\n"; }
+		if($item_type == 'node'){
+            if (isset($map->nodes[$item_name])) {
+                print $map->nodes[$item_name]->WriteConfig();
+                $ok=TRUE;
+            }
+		}
+		if($item_type == 'link') {
+            if(isset($map->links[$item_name])) {
+                print $map->links[$item_name]->WriteConfig();
+                $ok=TRUE;
+            }
+		}
+		
+		if (! $ok) { 
+            print "# the request item didn't exist. That's probably a bug.\n"; 
+        }
 		
 		exit();
 		break;
@@ -247,37 +250,40 @@ else
 		$map->ReadConfig($mapfile);
 
 		$link_name = $_REQUEST['link_name'];
-		$link_config = $_REQUEST['item_configtext'];
-                
-                $map->links[$link_name]->config_override=$link_config;
+		$link_config = fix_gpc_string($_REQUEST['item_configtext']);
+		                
+        $map->links[$link_name]->config_override = $link_config;
                 
 		$map->WriteConfig($mapfile);
-                // now clear and reload the map object, because the in-memory one is out of sync
-                // - we don't know what changes the user made here, so we just have to reload.
-                unset($map);
-                $map = new WeatherMap;
-                $map->context = 'editor';
-                $map->ReadConfig($mapfile);
+		// now clear and reload the map object, because the in-memory one is out of sync
+		// - we don't know what changes the user made here, so we just have to reload.
+		unset($map);
+		$map = new WeatherMap;
+		$map->context = 'editor';
+		$map->ReadConfig($mapfile);
 		break;
 
 	case "set_node_config":
 		$map->ReadConfig($mapfile);
 
 		$node_name = $_REQUEST['node_name'];
-		$node_config = $_REQUEST['item_configtext'];
+		$node_config = fix_gpc_string($_REQUEST['item_configtext']);
+		
+		if (true == function_exists('get_magic_quotes_gpc') && 1 == get_magic_quotes_gpc()) {
+			$link_config = stripslashes($link_config);
+		}
                 
-                $map->nodes[$node_name]->config_override=$node_config;
+        $map->nodes[$node_name]->config_override = $node_config;
                 
 		$map->WriteConfig($mapfile);
-                // now clear and reload the map object, because the in-memory one is out of sync
-                // - we don't know what changes the user made here, so we just have to reload.
-                unset($map);
-                $map = new WeatherMap;
-                $map->context = 'editor';
-                $map->ReadConfig($mapfile);
+		// now clear and reload the map object, because the in-memory one is out of sync
+		// - we don't know what changes the user made here, so we just have to reload.
+		unset($map);
+		$map = new WeatherMap;
+		$map->context = 'editor';
+		$map->ReadConfig($mapfile);
 		break;
 
-
 	case "set_node_properties":
 		$map->ReadConfig($mapfile);
 
@@ -370,7 +376,7 @@ else
 		$link_name = $_REQUEST['link_name'];
 
 
-		$map->links[$link_name]->width = intval($_REQUEST['link_width']);
+		$map->links[$link_name]->width = floatval($_REQUEST['link_width']);
 		$map->links[$link_name]->infourl[IN] = $_REQUEST['link_infourl'];
 		$map->links[$link_name]->infourl[OUT] = $_REQUEST['link_infourl'];
 		$urls = preg_split('/\s+/', $_REQUEST['link_hover'], -1, PREG_SPLIT_NO_EMPTY);
@@ -447,18 +453,19 @@ else
 			$map->background = stripslashes($_REQUEST['map_bgfile']);
 		}
 
-
 		$inheritables = array(
 			array('link','width','map_linkdefaultwidth'),
 		);
 
 		handle_inheritance($map, $inheritables);	
+		$map->links['DEFAULT']->width = intval($_REQUEST['map_linkdefaultwidth']);
+		$map->links['DEFAULT']->add_note("my_width", intval($_REQUEST['map_linkdefaultwidth']));
                 
 		$bwin = $_REQUEST['map_linkdefaultbwin'];
 		$bwout = $_REQUEST['map_linkdefaultbwout'];
 
-		$bwin_old = $map->defaultlink->max_bandwidth_in_cfg;
-		$bwout_old = $map->defaultlink->max_bandwidth_out_cfg;
+		$bwin_old = $map->links['DEFAULT']->max_bandwidth_in_cfg;
+		$bwout_old = $map->links['DEFAULT']->max_bandwidth_out_cfg;
 
 		if( ($bwin_old != $bwin) || ($bwout_old != $bwout) )
 		{
@@ -507,7 +514,7 @@ else
 		$map->ReadConfig($mapfile);
 
 		$param2 = $_REQUEST['param'];
-		$param2 = substr($param2,0,-2);
+		# $param2 = substr($param2,0,-2);
 		$newaction = 'add_link2';
               #  print $newaction;
 		$selected = 'NODE:'.$param2;
@@ -518,19 +525,20 @@ else
 		$map->ReadConfig($mapfile);
 		$a = $_REQUEST['param2'];
 		$b = $_REQUEST['param'];
-		$b = substr($b,0,-2);
+		# $b = substr($b,0,-2);
 		$log = "[$a -> $b]";
 
 		if($a != $b)
 		{
 			$newlink = new WeatherMapLink;
 			$newlink->Reset($map);
+			
 			$newlink->a = $map->nodes[$a];
 			$newlink->b = $map->nodes[$b];
 			
 			// $newlink->SetBandwidth($map->defaultlink->max_bandwidth_in_cfg, $map->defaultlink->max_bandwidth_out_cfg);
 						
-			$newlink->width = $map->defaultlink->width;
+			$newlink->width = $map->links['DEFAULT']->width;
 
 			// make sure the link name is unique. We can have multiple links between
 			// the same nodes, these days
@@ -540,6 +548,7 @@ else
 				$newlinkname .= "a";
 			}
 			$newlink->name = $newlinkname;
+			$newlink->defined_in = $map->configfile;
 			$map->links[$newlinkname] = $newlink;
 			array_push($map->seen_zlayers[$newlink->zorder], $newlink);
 
@@ -572,6 +581,20 @@ else
 		$map->WriteConfig($mapfile);
 		break;
 
+		
+	case "via_link":
+		$x = intval($_REQUEST['x']);
+		$y = intval($_REQUEST['y']);
+		$link_name = $_REQUEST['link_name'];
+
+		$map->ReadConfig($mapfile);
+
+		$map->links[$link_name]->vialist = array(array(0 =>$x, 1=>$y));
+
+		$map->WriteConfig($mapfile);
+		break;
+
+		
 	case "move_node":
 		$x = snap( intval($_REQUEST['x']),  $grid_snap_value);
 		$y = snap( intval($_REQUEST['y']), $grid_snap_value);
@@ -752,10 +775,11 @@ else
 		$node->name = $newnodename;
 		$node->template = "DEFAULT";
 		$node->Reset($map);
-		
+				
 		$node->x = $x;
 		$node->y = $y;
-		
+		$node->defined_in = $map->configfile;
+				
 		array_push($map->seen_zlayers[$node->zorder], $node);			
 		
 		// only insert a label if there's no LABEL in the DEFAULT node.
@@ -764,9 +788,10 @@ else
 		{
 			$node->label = "Node";
 		}
-
-		$map->nodes[$node->name] = $node;
 		
+		$map->nodes[$node->name] = $node;
+		$log = "added a node called $newnodename at $x,$y to $mapfile";
+						
 		$map->WriteConfig($mapfile);
 		break;
 
@@ -821,6 +846,8 @@ else
 		$node->name = $newnodename;
 		$node->x += 30;
 		$node->y += 30;
+                $node->defined_in = $mapfile;
+
 
 		$map->nodes[$newnodename] = $node;
 		array_push($map->seen_zlayers[$node->zorder], $node);
@@ -866,7 +893,7 @@ else
 <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
 <html xmlns="http://www.w3.org/1999/xhtml" lang="en" xml:lang="en">
 <head>
-<style>
+<style type="text/css">
 <?php
 		// if the cacti config was included properly, then 
 		// this will be non-empty, and we can unhide the cacti links in the Link Properties box
@@ -907,7 +934,7 @@ else
 <body id="mainview">
   <div id="toolbar">
 	<ul>
-           <li class="tb_active" id="tb_newfile">Change<br />File</li>
+          <li class="tb_active" id="tb_newfile">Change<br />File</li>
 	  <li class="tb_active" id="tb_addnode">Add<br />Node</li>
 	  <li class="tb_active" id="tb_addlink">Add<br />Link</li>
 	  <li class="tb_active" id="tb_poslegend">Position<br />Legend</li>
@@ -917,13 +944,13 @@ else
 	  <li class="tb_active" id="tb_colours">Manage<br />Colors</li>
 	  <li class="tb_active" id="tb_manageimages">Manage<br />Images</li>
 	  <li class="tb_active" id="tb_prefs">Editor<br />Settings</li>
-        <li class="tb_coords" id="tb_coords">Position<br />---, ---</li>
+          <li class="tb_coords" id="tb_coords">Position<br />---, ---</li>
 	  <li class="tb_help"><span id="tb_help">or click a Node or Link to edit it's properties</span></li>
 	</ul>
   </div>
   <form action="editor.php" method="post" name="frmMain">
 	<div align="center" id="mainarea">
-		<input type="hidden" name="plug" value="<?php echo ($fromplug==TRUE ? 1 : 0) ?>">
+		<input type="hidden" name="plug" value="<?php echo ($fromplug==TRUE ? 1 : 0) ?>" />
 	 <input style="display:none" type="image"
 	  src="<?php echo  $imageurl; ?>" id="xycapture" /><img src=
 	  "<?php echo  $imageurl; ?>" id="existingdata" alt="Weathermap" usemap="#weathermap_imap"
@@ -937,7 +964,6 @@ else
 	  <a target="configwindow" href="?<?php echo ($fromplug==TRUE ? 'plug=1&amp;':''); ?>action=show_config&amp;mapname=<?php echo  $mapname ?>">See config</a></p>
 	<pre><?php echo  $log ?></pre>
 	  </div>
-	   <map name="weathermap_imap">
 <?php        	
 	// we need to draw and throw away a map, to get the
 	// dimensions for the imagemap. Oh well.
@@ -945,13 +971,14 @@ else
 	$map->htmlstyle='editor';
 	$map->PreloadMapHTML();
 
-	print $map->imap->subHTML("LEGEND:");
-	print $map->imap->subHTML("TIMESTAMP");
-	print $map->imap->subHTML("NODE:");
-	print $map->imap->subHTML("LINK:");
+	print $map->SortedImagemap("weathermap_imap");
+
+	#print $map->imap->subHTML("LEGEND:");
+	#print $map->imap->subHTML("TIMESTAMP");
+	#print $map->imap->subHTML("NODE:");
+	#print $map->imap->subHTML("LINK:");
                 
 ?>
-		   </map>
 	</div><!-- Node Properties -->
 
 	<div id="dlgNodeProperties" class="dlgProperties">
@@ -1119,7 +1146,8 @@ else
 			  <td><a class="dlgTitlebar" id="link_delete">Delete
 			  Link</a><a class="dlgTitlebar" id="link_edit">Edit</a><a
                             class="dlgTitlebar" id="link_vert">Vert</a><a
-                            class="dlgTitlebar" id="link_horiz">Horiz</a>                          
+                            class="dlgTitlebar" id="link_horiz">Horiz</a><a 
+							class="dlgTitlebar" id="link_via">Via</a> 
                         </td>
 			</tr>
 		  </table>
@@ -1161,12 +1189,12 @@ else
 
 		<tr>
 			<th>Default Link Width</th>
-			<td><input name="map_linkdefaultwidth" size="6" type="text" value="<?php echo  $map->defaultlink->width ?>" /> pixels</td>
+			<td><input name="map_linkdefaultwidth" size="6" type="text" value="<?php echo  $map->links['DEFAULT']->width ?>" /> pixels</td>
 		  </tr>
 
 		<tr>
 			<th>Default Link Bandwidth</th>
-			<td><input name="map_linkdefaultbwin" size="6" type="text" value="<?php echo  $map->defaultlink->max_bandwidth_in_cfg ?>" /> bit/sec in, <input name="map_linkdefaultbwout" size="6" type="text" value="<?php echo  $map->defaultlink->max_bandwidth_out_cfg ?>" /> bit/sec out</td>
+			<td><input name="map_linkdefaultbwin" size="6" type="text" value="<?php echo  $map->links['DEFAULT']->max_bandwidth_in_cfg ?>" /> bit/sec in, <input name="map_linkdefaultbwout" size="6" type="text" value="<?php echo  $map->links['DEFAULT']->max_bandwidth_out_cfg ?>" /> bit/sec out</td>
 		  </tr>
 
 
@@ -1234,9 +1262,9 @@ else
 		  <tr>
 			<th>Link Labels</th>
 			<td><select id="mapstyle_linklabels" name="mapstyle_linklabels">
-			  <option <?php echo ($map->defaultlink->labelstyle=='bits' ? 'selected' : '') ?> value="bits">Bits/sec</option>
-			  <option <?php echo ($map->defaultlink->labelstyle=='percent' ? 'selected' : '') ?> value="percent">Percentage</option>
-			  <option <?php echo ($map->defaultlink->labelstyle=='none' ? 'selected' : '') ?> value="none">None</option>
+			  <option <?php echo ($map->links['DEFAULT']->labelstyle=='bits' ? 'selected' : '') ?> value="bits">Bits/sec</option>
+			  <option <?php echo ($map->links['DEFAULT']->labelstyle=='percent' ? 'selected' : '') ?> value="percent">Percentage</option>
+			  <option <?php echo ($map->links['DEFAULT']->labelstyle=='none' ? 'selected' : '') ?> value="none">None</option>
 			</select></td>
 		  </tr>
 		  <tr>
@@ -1249,17 +1277,17 @@ else
 		  <tr>
 			<th>Arrow Style</th>
 			<td><select name="mapstyle_arrowstyle">
-			  <option <?php echo ($map->defaultlink->arrowstyle=='classic' ? 'selected' : '') ?> value="classic">Classic</option>
-			  <option <?php echo ($map->defaultlink->arrowstyle=='compact' ? 'selected' : '') ?> value="compact">Compact</option>
+			  <option <?php echo ($map->links['DEFAULT']->arrowstyle=='classic' ? 'selected' : '') ?> value="classic">Classic</option>
+			  <option <?php echo ($map->links['DEFAULT']->arrowstyle=='compact' ? 'selected' : '') ?> value="compact">Compact</option>
 			</select></td>
 		  </tr>
 		  <tr>
 			<th>Node Font</th>
-			<td><?php echo get_fontlist($map,'mapstyle_nodefont',$map->defaultnode->labelfont); ?></td>
+			<td><?php echo get_fontlist($map,'mapstyle_nodefont',$map->nodes['DEFAULT']->labelfont); ?></td>
 		  </tr>
 		  <tr>
 			<th>Link Label Font</th>
-			<td><?php echo get_fontlist($map,'mapstyle_linkfont',$map->defaultlink->bwfont); ?></td>
+			<td><?php echo get_fontlist($map,'mapstyle_linkfont',$map->links['DEFAULT']->bwfont); ?></td>
 		  </tr>
 		  <tr>
 			<th>Legend Font</th>
@@ -1267,7 +1295,7 @@ else
 		  </tr>
 		  <tr>
 			<th>Font Samples:</th>
-			<td><div class="fontsamples" ><img src="?action=font_samples&mapname=<?php echo  $mapname?>" /></div><br />(Drawn using your PHP install)</td>
+			<td><div class="fontsamples" ><img alt="Sample of defined fonts" src="?action=font_samples&mapname=<?php echo  $mapname?>" /></div><br />(Drawn using your PHP install)</td>
 		  </tr>
 		</table>
 	  </div>
