@@ -4,6 +4,9 @@
 //     filename.rrd:ds_in:ds_out
 //     filename.rrd:ds_in:ds_out
 //
+
+include_once(dirname(__FILE__)."/../ds-common.php");
+
 class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 
 	function Init(&$map)
@@ -17,8 +20,7 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 #		else
 #		{
 			if($map->context=='cacti')
-			{
-				
+			{			
 				debug("RRD DS: path_rra is ".$config["rra_path"]." - your rrd pathname must be exactly this to use poller_output\n");
 				// save away a couple of useful global SET variables
 				$map->add_hint("cacti_path_rra",$config["rra_path"]);
@@ -92,6 +94,7 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 					
 					$worst_time = time() - 8*60;
 					$result = db_fetch_row($SQL);
+					// OK, the straightforward query for data failed, let's work out why, and add the new data source if necessary
 					if(!isset($result['id']))
 					{
 						debug("RRD ReadData: poller_output - Adding new weathermap_data row for $db_rrdname:".$dsnames[$dir]."\n");
@@ -106,11 +109,11 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 							}
 							if(count($fields) > 0)
 							{
-								warn("RRD ReadData: poller_output: ".$dsnames[$dir]." is not a valid DS name for $db_rrdname - valid names are: ".join(", ",$fields)."\n");
+								warn("RRD ReadData: poller_output: ".$dsnames[$dir]." is not a valid DS name for $db_rrdname - valid names are: ".join(", ",$fields)." [WMRRD07]\n");
 							}
 							else
 							{
-								warn("RRD ReadData: poller_output: $db_rrdname is not a valid RRD filename within this Cacti install. <path_rra> is $path_rra\n");
+								warn("RRD ReadData: poller_output: $db_rrdname is not a valid RRD filename within this Cacti install. <path_rra> is $path_rra [WMRRD08]\n");
 							}
 						}
 						else
@@ -119,6 +122,7 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 							// Include the local_data_id as well, to make life easier in poller_output
 							// (and to allow the cacti: DS plugin to use the same table, too)
 							$SQLins = "insert into weathermap_data (rrdfile, data_source_name, sequence, local_data_id) values ('".mysql_real_escape_string($db_rrdname)."','".mysql_real_escape_string($dsnames[$dir])."', 0,".$result['local_data_id'].")";
+							debug("RRD ReadData: poller_output - Adding new weathermap_data row for data source ID ".$result['local_data_id']."\n");
 							db_execute($SQLins);
 						}
 					}
@@ -140,65 +144,24 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 						// now, we can use the local_data_id to get some other useful info
 						// first, see if the weathermap_data entry *has* a local_data_id. If not, we need to update this entry.
 						$ldi = 0;
-						if(!isset($result['local_data_id']))
+						if(!isset($result['local_data_id']) || $result['local_data_id']==0)
 						{
 							$r2 = db_fetch_row($SQLcheck);
 							if(isset($r2['local_data_id']))
 							{
-								debug("RRD ReadData: updated local_data_id\n");
+								$ldi = $r2['local_data_id'];
+								debug("RRD ReadData: updated  local_data_id for wmdata.id=" . $result['id'] . "to $ldi\n");
 								// put that in now, so that we can skip this step next time
 								db_execute("update weathermap_data set local_data_id=".$r2['local_data_id']." where id=".$result['id']);
-								$ldi = $r2['local_data_id'];
+								
 							}
 						}
 						else
 						{
 							$ldi = $result['local_data_id'];
 						}
-						if($ldi>0)
-						{
-							$set_speed = intval($item->get_hint("cacti_use_ifspeed"));
-							
-							$r3 = db_fetch_assoc(sprintf("select data_local.host_id, field_name,field_value from data_local,host_snmp_cache where data_local.id=%d and data_local.host_id=host_snmp_cache.host_id and data_local.snmp_index=host_snmp_cache.snmp_index and data_local.snmp_query_id=host_snmp_cache.snmp_query_id",$ldi));
-							foreach ($r3 as $vv)
-							{
-								$vname = "cacti_".$vv['field_name'];
-								$item->add_note($vname,$vv['field_value']);
-							}
-							
-							if($set_speed != 0)
-							{
-								# $item->max_bandwidth_in = $vv['field_value'];
-								# $item->max_bandwidth_out = $vv['field_value'];
-								
-								$ifSpeed = intval($item->get_note('cacti_ifSpeed'));
-								$ifHighSpeed = intval($item->get_note('cacti_ifHighSpeed'));
-								$speed = 0;
-								if($ifSpeed > 0) $speed = $ifSpeed;
-								# see https://lists.oetiker.ch/pipermail/mrtg/2004-November/029312.html
-								if($ifHighSpeed > 20) $speed = $ifHighSpeed."M";
-								
-								if($speed >0)
-								{
-									// might need to dust these off for php4...
-									if($item->my_type() == 'NODE') 
-									{
-										$map->nodes[$item->name]->max_bandwidth_in = $speed;
-										$map->nodes[$item->name]->max_bandwidth_out = $speed;
-									}
-									if($item->my_type() == 'LINK') 
-									{
-										$map->links[$item->name]->max_bandwidth_in = $speed;
-										$map->links[$item->name]->max_bandwidth_out = $speed;
-									}
-								}
-							}
-							
-							if(isset($vv['host_id'])) $item->add_note("cacti_host_id",intval($vv['host_id']));
-							
-							$r4 = db_fetch_row(sprintf("SELECT DISTINCT graph_templates_item.local_graph_id,title_cache FROM graph_templates_item,graph_templates_graph,data_template_rrd WHERE data_template_rrd.id=task_item_id and graph_templates_graph.local_graph_id = graph_templates_item.local_graph_id and local_data_id=%d LIMIT 1",$ldi));
-							if(isset($r4['local_graph_id'])) $item->add_note("cacti_graph_id",intval($r4['local_graph_id']));
-						}
+						
+						if($ldi>0) UpdateCactiData($item, $ldi);
 					}
 				}				
 				else
@@ -327,7 +290,7 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 			{			
 				foreach ($lines as $line)
 				{
-					if(preg_match('/^(IN|OUT)\s(\-?\d+[\.,]?\d*e?[+-]?\d*:?)$/i', $line, $matches))
+					if(preg_match('/^\'(IN|OUT)\s(\-?\d+[\.,]?\d*e?[+-]?\d*:?)\'$/i', $line, $matches))
 					{
 						debug("MATCHED: ".$matches[1]." ".$matches[2]."\n");
 						if($matches[1]=='IN') $data[IN] = floatval($matches[2]);
@@ -343,7 +306,7 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 			}
 			else
 			{
-				warn("Not enough output from RRDTool.\n");
+				warn("Not enough output from RRDTool. [WMRRD09]\n");
 			}
 		}
 		else
@@ -587,7 +550,7 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 			$use_poller_output=0;
 			if($nowarn_po_agg==0)
 			{
-				warn("Can't use poller_output for rrd-aggregated data - disabling rrd_use_poller_output\n");
+				warn("Can't use poller_output for rrd-aggregated data - disabling rrd_use_poller_output [WMRRD10]\n");
 			}
 		}
 		
@@ -635,8 +598,20 @@ class WeatherMapDataSource_rrd extends WeatherMapDataSource {
 			}
 		}
 
-		if($data[IN] !== NULL) $data[IN] = $data[IN] * $multiplier;
-		if($data[OUT] !== NULL) $data[OUT] = $data[OUT] * $multiplier;
+                // if the Locale says that , is the decimal point, then rrdtool
+                // will honour it. However, floatval() doesn't, so let's replace
+                // any , with . (there are never thousands separators, luckily)
+                //
+		if($data[IN] !== NULL)
+                {
+                    $data[IN] = floatval(str_replace(",",".",$data[IN]));
+                    $data[IN] = $data[IN] * $multiplier;                    
+                }
+		if($data[OUT] !== NULL) 
+                {
+                    $data[OUT] = floatval(str_replace(",",".",$data[OUT]));
+                    $data[OUT] = $data[OUT] * $multiplier;
+                }
 				
 		debug ("RRD ReadData: Returning (".($data[IN]===NULL?'NULL':$data[IN]).",".($data[OUT]===NULL?'NULL':$data[OUT]).",$data_time)\n");
 		
