@@ -9,8 +9,8 @@ class WeatherMapDataSource_static extends WeatherMapDataSource {
 
 	function Recognise($targetstring)
 	{
-		if( preg_match("/^static:(\d+\.?\d*[KMGT]?):(\d+\.?\d*[KMGT]?)$/",$targetstring,$matches) || 
-			preg_match("/^static:(\d+\.?\d*[KMGT]?)$/",$targetstring,$matches) )
+		if( preg_match("/^static:(\-?\d+\.?\d*[KMGT]?):(\-?\d+\.?\d*[KMGT]?)$/",$targetstring,$matches) || 
+			preg_match("/^static:(\-?\d+\.?\d*[KMGT]?)$/",$targetstring,$matches) )
 		{
 			return TRUE;
 		}
@@ -26,14 +26,14 @@ class WeatherMapDataSource_static extends WeatherMapDataSource {
 		$outbw = NULL;
 		$data_time=0;
 
-		if(preg_match("/^static:(\d+\.?\d*[KMGT]*):(\d+\.?\d*[KMGT]*)$/",$targetstring,$matches))
+		if(preg_match("/^static:(\-?\d+\.?\d*[KMGT]*):(\-?\d+\.?\d*[KMGT]*)$/",$targetstring,$matches))
 		{
 			$inbw = unformat_number($matches[1]);
 			$outbw = unformat_number($matches[2]);
 			$data_time = time();
 		}
 
-		if(preg_match("/^static:(\d+\.?\d*[KMGT]*)$/",$targetstring,$matches))
+		if(preg_match("/^static:(\-?\d+\.?\d*[KMGT]*)$/",$targetstring,$matches))
 		{
 			$inbw = unformat_number($matches[1]);
 			$outbw = unformat_number($matches[1]);
