@@ -18,7 +18,7 @@ function disabled_plugin_weathermap_version()
 
 function weathermap_version () {
 	return array( 	'name'    	=> 'weathermap',
-		'version'       => '0.96a',
+		'version'       => '0.97a',
 		'longname'      => 'PHP Network Weathermap',
 		'author'        => 'Howard Jones',
 		'homepage'      => 'http://www.network-weathermap.com/',
@@ -105,10 +105,14 @@ function weathermap_page_head()
 	
 	// Add in a Media RSS link on the thumbnail view
 	// - format isn't quite right, so it's disabled for now.
-//	if(preg_match('/plugins\/weathermap\/weathermap\-cacti\-plugin\.php/',$_SERVER['REQUEST_URI'] ,$matches))
-//	{
-//		print '<link id="media-rss" title="My Network Weathermaps" rel="alternate" href="?action=mrss" type="application/rss+xml">';
-//	}
+	//	if(preg_match('/plugins\/weathermap\/weathermap\-cacti\-plugin\.php/',$_SERVER['REQUEST_URI'] ,$matches))
+	//	{
+	//		print '<link id="media-rss" title="My Network Weathermaps" rel="alternate" href="?action=mrss" type="application/rss+xml">';
+	//	}
+	if(preg_match('/plugins\/weathermap\//',$_SERVER['REQUEST_URI'] ,$matches))
+    {
+		print '<LINK rel="stylesheet" type="text/css" media="screen" href="weathermap-cacti-plugin.css">';
+	}
 }
 
 function weathermap_page_title( $t )
@@ -143,7 +147,7 @@ function weathermap_top_graph_refresh($refresh)
 		return $refresh;
 
 	// if we're cycling maps, then we want to handle reloads ourselves, thanks
-	if($_REQUEST["action"] == 'viewmapcycle')
+	if(isset($_REQUEST["action"]) && $_REQUEST["action"] == 'viewmapcycle')
 	{
 		return(86400);
 	}
@@ -212,6 +216,18 @@ function weathermap_config_settings () {
 		),
 	),
 
+	"weathermap_all_tab" => array(
+		"friendly_name" => "Show 'All' Tab",
+		"description" => "When using groups, add an 'All Maps' tab to the tab bar.",
+		"method" => "drop_array",
+		"array" => array(0=>"No (default)",1=>"Yes")
+		),
+	"weathermap_map_selector" => array(
+		"friendly_name" => "Show Map Selector",
+		"description" => "Show a combo-box map selector on the full-screen map view.",
+		"method" => "drop_array",
+		"array" => array(0=>"No",1=>"Yes (default)")
+		),
 	"weathermap_quiet_logging" => array(
 		"friendly_name" => "Quiet Logging",
 		"description" => "By default, even in LOW level logging, Weathermap logs normal activity. This makes it REALLY log only errors in LOW mode.",
@@ -238,8 +254,9 @@ function weathermap_setup_table () {
 	
 	// only bother with all this if it's a new install, a new version, or we're in a development version
 	// - saves a handful of db hits per request!
-	if( ($dbversion=="") || ( ($dbversion != $myversion) && !preg_match("/dev$/",$myversion) ) )
+	if( ($dbversion=="") || (preg_match("/dev$/",$myversion)) || ($dbversion != $myversion) )
 	{
+		# cacti_log("Doing setup_table() \n",true,"WEATHERMAP");
 		$sql = "show tables";
 		$result = db_fetch_assoc($sql) or die (mysql_error());
 
@@ -258,6 +275,7 @@ function weathermap_setup_table () {
 			$sql[] = "CREATE TABLE weathermap_maps (
 				id int(11) NOT NULL auto_increment,
 				sortorder int(11) NOT NULL default 0,
+				group_id int(11) NOT NULL default 1,
 				active set('on','off') NOT NULL default 'on',
 				configfile text NOT NULL,
 				imagefile text NOT NULL,
@@ -280,6 +298,7 @@ function weathermap_setup_table () {
 			$found_so = false;	$found_fh = false;
 			$found_wc = false;	$found_cf = false;
 			$found_96changes = false;
+			$found_96bchanges = false;
 			
 			while ($row = mysql_fetch_array($result, MYSQL_ASSOC)) {
 				if ($row['Field'] == 'sortorder') $found_so = true;
@@ -288,6 +307,7 @@ function weathermap_setup_table () {
 				if ($row['Field'] == 'config') $found_cf = true;
 				
 				if ($row['Field'] == 'thumb_width') $found_96changes = true;
+				if ($row['Field'] == 'group_id') $found_96bchanges = true;
 			}
 			if (!$found_so) $sql[] = "alter table weathermap_maps add sortorder int(11) NOT NULL default 0 after id";
 			if (!$found_fh) $sql[] = "alter table weathermap_maps add filehash varchar(40) NOT NULL default '' after titlecache";		
@@ -300,6 +320,11 @@ function weathermap_setup_table () {
 				$sql[] = "alter table weathermap_maps add schedule varchar(32) NOT NULL default '*' after thumb_height";
 				$sql[] = "alter table weathermap_maps add archiving set('on','off') NOT NULL default 'off' after schedule";
 			}
+			if (!$found_96bchanges)
+			{
+				$sql[] = "alter table weathermap_maps add group_id int(11) NOT NULL default 1 after sortorder";
+				$sql[] = "ALTER TABLE `weathermap_settings` ADD `groupid` INT NOT NULL DEFAULT '0' AFTER `mapid`";
+			}
 		}
 
 		$sql[] = "update weathermap_maps set filehash=LEFT(MD5(concat(id,configfile,rand())),20) where filehash = '';";
@@ -310,11 +335,23 @@ function weathermap_setup_table () {
 				mapid int(11) NOT NULL default '0'
 			) TYPE=MyISAM;";
 		}
+
+		
+		if (!in_array('weathermap_groups', $tables)) {
+			$sql[] = "CREATE TABLE  weathermap_groups (
+				`id` INT(11) NOT NULL auto_increment,
+				`name` VARCHAR( 128 ) NOT NULL default '',
+				`sortorder` INT(11) NOT NULL default 0,
+				PRIMARY KEY (id)
+				) TYPE=MyISAM;";
+			$sql[] = "INSERT INTO weathermap_groups (id,name,sortorder) VALUES (1,'Weathermaps',1)";
+		}
 		
 		if (!in_array('weathermap_settings', $tables)) {
 			$sql[] = "CREATE TABLE weathermap_settings (
 				id int(11) NOT NULL auto_increment,
 				mapid int(11) NOT NULL default '0',
+				groupid int(11) NOT NULL default '0',
 				optname varchar(128) NOT NULL default '',
 				optvalue varchar(128) NOT NULL default '',
 				PRIMARY KEY  (id)
@@ -341,6 +378,8 @@ function weathermap_setup_table () {
 			{
 				$sql[] = "alter table weathermap_data add local_data_id int(11) NOT NULL default 0 after sequence";
 				$sql[] = "alter table weathermap_data add index ( `local_data_id` )";
+				# if there is existing data without a local_data_id, ditch it
+				$sql[] = "delete from weathermap_data";
 			}
 		}
 
@@ -388,6 +427,18 @@ function weathermap_setup_table () {
 			$sql[] = "replace into settings values('weathermap_thumbsize',250)";
 		}
 
+		$ms = read_config_option("weathermap_map_selector");
+		if($ms == '' or intval($ms) < 0 or intval($ms) > 1)
+		{
+			$sql[] = "replace into settings values('weathermap_map_selector',1)";
+		}
+
+		$at = read_config_option("weathermap_all_tab");
+		if($at == '' or intval($at) < 0 or intval($at) > 1)
+		{
+			$sql[] = "replace into settings values('weathermap_all_tab',0)";
+		}
+
 		// update the version, so we can skip this next time
 		$sql[] = "replace into settings values('weathermap_db_version','$myversion')";
 		
@@ -396,10 +447,15 @@ function weathermap_setup_table () {
 
 		if (!empty($sql)) {
 			for ($a = 0; $a < count($sql); $a++) {
+				# cacti_log("Executing SQL: ".$sql[$a]."\n",true,"WEATHERMAP");
 				$result = db_execute($sql[$a]);
 			}
 		}
 	}
+	else
+	{
+		# cacti_log("Skipping SQL updates\n",true,"WEATHERMAP");
+	}
 }
 
 function weathermap_config_arrays () {
@@ -414,6 +470,7 @@ function weathermap_config_arrays () {
 		$user_auth_realms[43]='Plugin -> Weathermap: View';
 		$user_auth_realm_filenames['weathermap-cacti-plugin.php'] = 43;
 		$user_auth_realm_filenames['weathermap-cacti-plugin-mgmt.php'] = 42;
+#		$user_auth_realm_filenames['weathermap-cacti-plugin-mgmt-groups.php'] = 42;
 	}
 	// if there is support for custom graph tree types, then register ourselves
 	if(isset($tree_item_handlers))
@@ -424,7 +481,13 @@ function weathermap_config_arrays () {
 					"edit" => "weathermap_tree_item_edit");
 	}
 
-	$menu["Management"]['plugins/weathermap/weathermap-cacti-plugin-mgmt.php'] = "Weathermaps";
+	$wm_menu = array(
+		'plugins/weathermap/weathermap-cacti-plugin-mgmt.php' => "Weathermaps",
+		'plugins/weathermap/weathermap-cacti-plugin-mgmt-groups.php' => "Groups"
+	);
+	
+	$menu["Management"]['plugins/weathermap/weathermap-cacti-plugin-mgmt.php'] = $wm_menu;
+	
 }
 
 function weathermap_tree_item_render($leaf)
@@ -565,15 +628,24 @@ function weathermap_draw_navigation_text ($nav) {
 	$nav["weathermap-cacti-plugin-mgmt.php:delete_map"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
 	$nav["weathermap-cacti-plugin-mgmt.php:move_map_down"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
 	$nav["weathermap-cacti-plugin-mgmt.php:move_map_up"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+	$nav["weathermap-cacti-plugin-mgmt.php:move_group_down"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+	$nav["weathermap-cacti-plugin-mgmt.php:move_group_up"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+	$nav["weathermap-cacti-plugin-mgmt.php:group_form"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+	$nav["weathermap-cacti-plugin-mgmt.php:group_update"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
 	$nav["weathermap-cacti-plugin-mgmt.php:activate_map"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
 	$nav["weathermap-cacti-plugin-mgmt.php:deactivate_map"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
 	$nav["weathermap-cacti-plugin-mgmt.php:rebuildnow"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
 	$nav["weathermap-cacti-plugin-mgmt.php:rebuildnow2"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
-	
+
+	$nav["weathermap-cacti-plugin-mgmt.php:chgroup"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+	$nav["weathermap-cacti-plugin-mgmt.php:chgroup_update"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+	$nav["weathermap-cacti-plugin-mgmt.php:groupadmin"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+	$nav["weathermap-cacti-plugin-mgmt.php:groupadmin_delete"] = array("title" => "Weathermap Management", "mapping" => "index.php:", "url" => "weathermap-cacti-plugin-mgmt.php", "level" => "1");
+		
 	return $nav;
 }
 
-function weathermap_poller_output ($rrd_update_array) {
+function weathermap_poller_output($rrd_update_array) {
 	global $config;
 	// global $weathermap_debugging;
 
@@ -587,7 +659,7 @@ function weathermap_poller_output ($rrd_update_array) {
 	
 	// $requiredlist = db_fetch_assoc("select distinct weathermap_data.*, data_template_data.local_data_id, data_template_rrd.data_source_type_id from weathermap_data, data_template_data, data_template_rrd where weathermap_data.rrdfile=data_template_data.data_source_path and data_template_rrd.local_data_id=data_template_data.local_data_id");
 	// new version works with *either* a local_data_id or rrdfile in the weathermap_data table, and returns BOTH
-	$requiredlist = db_fetch_assoc("select distinct weathermap_data.id, weathermap_data.last_value, weathermap_data.last_time, weathermap_data.data_source_name, data_template_data.data_source_path, data_template_data.local_data_id, data_template_rrd.data_source_type_id from weathermap_data, data_template_data, data_template_rrd where ((weathermap_data.rrdfile=data_template_data.data_source_path and weathermap_data.rrdfile <> '') or (weathermap_data.local_data_id=data_template_data.local_data_id and weathermap_data.local_data_id>0)) and data_template_rrd.local_data_id=data_template_data.local_data_id;");
+	$requiredlist = db_fetch_assoc("select distinct weathermap_data.id, weathermap_data.last_value, weathermap_data.last_time, weathermap_data.data_source_name, data_template_data.data_source_path, data_template_data.local_data_id, data_template_rrd.data_source_type_id from weathermap_data, data_template_data, data_template_rrd where weathermap_data.local_data_id=data_template_data.local_data_id and data_template_rrd.local_data_id=data_template_data.local_data_id and weathermap_data.local_data_id<>0;");
 	
 	$path_rra = $config["rra_path"];
 	
@@ -618,7 +690,7 @@ function weathermap_poller_output ($rrd_update_array) {
 				
 	    if($logging >= POLLER_VERBOSITY_DEBUG) cacti_log("WM poller_output: Looking for $file ($local_data_id) (".$required['data_source_path'].")\n",true,"WEATHERMAP");
 		
-		if( is_array($rrd_update_array[$file]['times']) && isset( $rrd_update_array{$file}['times'][key($rrd_update_array[$file]['times'])]{$dsname} ) )
+		if( isset($rrd_update_array[$file]) && is_array($rrd_update_array[$file]) && isset($rrd_update_array[$file]['times']) && is_array($rrd_update_array[$file]['times']) && isset( $rrd_update_array{$file}['times'][key($rrd_update_array[$file]['times'])]{$dsname} ) )
 		{
 			$value = $rrd_update_array{$file}['times'][key($rrd_update_array[$file]['times'])]{$dsname};
 			$time = key($rrd_update_array[$file]['times']);
@@ -702,7 +774,7 @@ function weathermap_poller_output ($rrd_update_array) {
 	return $rrd_update_array;
 }
 
-function weathermap_poller_bottom () {
+function weathermap_poller_bottom() {
 	global $config;
 	global $weathermap_debugging, $WEATHERMAP_VERSION;
 
