@@ -243,17 +243,38 @@ default:
 	print "<div id=\"overDiv\" style=\"position:absolute; visibility:hidden; z-index:1000;\"></div>\n";
 	print "<script type=\"text/javascript\" src=\"overlib.js\"><!-- overLIB (c) Erik Bosrup --></script> \n";
 
+	$group_id = -1;
+	if( isset($_REQUEST['group_id']) && (is_numeric($_REQUEST['group_id']) ) )
+	{
+		$group_id = intval($_REQUEST['group_id']);
+		$_SESSION['wm_last_group'] = $group_id;
+	}
+	else
+	{
+		if(isset($_SESSION['wm_last_group']))
+		{
+			$group_id = intval($_SESSION['wm_last_group']);
+		}
+	}
+
+	$tabs = weathermap_get_valid_tabs();
+	$tab_ids = array_keys($tabs);
+	if( ($group_id == -1) && (sizeof($tab_ids)>0))
+	{
+		$group_id = $tab_ids[0];
+	}
+	
 	if(read_config_option("weathermap_pagestyle") == 0)
 	{
-		weathermap_thumbview();
+		weathermap_thumbview($group_id);
 	}
 	if(read_config_option("weathermap_pagestyle") == 1)
 	{
-		weathermap_fullview(FALSE,FALSE);
+		weathermap_fullview(FALSE,FALSE,$group_id);
 	}
 	if(read_config_option("weathermap_pagestyle") == 2)
 	{
-		weathermap_fullview(FALSE, TRUE);
+		weathermap_fullview(FALSE, TRUE, $group_id);
 	}
 
 	weathermap_versionbox();
@@ -279,6 +300,7 @@ function weathermap_singleview($mapid)
 	$userid = (isset($_SESSION["sess_user_id"]) ? intval($_SESSION["sess_user_id"]) : 1);
 	$map = db_fetch_assoc("select weathermap_maps.* from weathermap_auth,weathermap_maps where weathermap_maps.id=weathermap_auth.mapid and active='on' and (userid=".$userid." or userid=0) and weathermap_maps.id=".$mapid);
 
+
 	if(sizeof($map))
 	{
  		# print do_hook_function ('weathermap_page_top', array($map[0]['id'], $map[0]['titlecache']) );
@@ -288,6 +310,8 @@ function weathermap_singleview($mapid)
 		$maptitle = $map[0]['titlecache'];
 		if($maptitle == '') $maptitle= "Map for config file: ".$map[0]['configfile'];
 
+		weathermap_mapselector($mapid);
+
 		html_graph_start_box(1,true);
 ?>
 <tr bgcolor="<?php print $colors["panel"];?>"><td><table width="100%" cellpadding="0" cellspacing="0"><tr><td class="textHeader" nowrap><?php print $maptitle; 
@@ -352,18 +376,22 @@ function weathermap_show_manage_tab()
 	}
 }
 
-function weathermap_thumbview()
+function weathermap_thumbview($limit_to_group = -1)
 {
 	global $colors;
 
 	$userid = (isset($_SESSION["sess_user_id"]) ? intval($_SESSION["sess_user_id"]) : 1);
-	$maplist = db_fetch_assoc( "select distinct weathermap_maps.* from weathermap_auth,weathermap_maps where weathermap_maps.id=weathermap_auth.mapid and active='on' and (userid=".$userid." or userid=0) order by sortorder, id");
-
-
+	$maplist_SQL = "select distinct weathermap_maps.* from weathermap_auth,weathermap_maps where weathermap_maps.id=weathermap_auth.mapid and active='on' and ";
+	if($limit_to_group >0) $maplist_SQL .= " weathermap_maps.group_id=".$limit_to_group." and ";
+	$maplist_SQL .= " (userid=".$userid." or userid=0) order by sortorder, id";
+	
+	$maplist = db_fetch_assoc( $maplist_SQL );
+	
+	// if there's only one map, ignore the thumbnail setting and show it fullsize
 	if(sizeof($maplist) == 1)
 	{
 		$pagetitle = "Network Weathermap";
-		weathermap_fullview(FALSE,FALSE);
+		weathermap_fullview(FALSE,FALSE, $limit_to_group);
 	}
 	else
 	{
@@ -388,6 +416,7 @@ function weathermap_thumbview()
 		html_graph_end_box();
 		$showlivelinks = intval(read_config_option("weathermap_live_view"));
 
+		weathermap_tabs($limit_to_group);
 		$i = 0;
 		if (sizeof($maplist) > 0)
 		{
@@ -397,7 +426,7 @@ function weathermap_thumbview()
 
 			$imageformat = strtolower(read_config_option("weathermap_output_format"));
 
-			html_graph_start_box(1,true);
+			html_graph_start_box(1,false);
 			print "<tr><td class='wm_gallery'>";
 			foreach ($maplist as $map) {
 				$i++;
@@ -437,18 +466,21 @@ function weathermap_thumbview()
 	}
 }
 
-function weathermap_fullview($cycle=FALSE, $firstonly=FALSE)
+function weathermap_fullview($cycle=FALSE, $firstonly=FALSE, $limit_to_group = -1)
 {
 	global $colors;
 
 	$_SESSION['custom']=false;
 
 	$userid = (isset($_SESSION["sess_user_id"]) ? intval($_SESSION["sess_user_id"]) : 1);
-	$query = "select distinct weathermap_maps.* from weathermap_auth,weathermap_maps where weathermap_maps.id=weathermap_auth.mapid and active='on' and (userid=".$userid." or userid=0) order by sortorder, id";
+	# $query = "select distinct weathermap_maps.* from weathermap_auth,weathermap_maps where weathermap_maps.id=weathermap_auth.mapid and active='on' and (userid=".$userid." or userid=0) order by sortorder, id";
+	$maplist_SQL = "select distinct weathermap_maps.* from weathermap_auth,weathermap_maps where weathermap_maps.id=weathermap_auth.mapid and active='on' and ";
+	if($limit_to_group >0) $maplist_SQL .= " weathermap_maps.group_id=".$limit_to_group." and ";
+	$maplist_SQL .= " (userid=".$userid." or userid=0) order by sortorder, id";
 
-	if($firstonly) { $query .= " LIMIT 1"; }
+	if($firstonly) { $maplist_SQL .= " LIMIT 1"; }
 
-	$maplist = db_fetch_assoc( $query );
+	$maplist = db_fetch_assoc( $maplist_SQL );
 	html_graph_start_box(2,true);
 
 	if(sizeof($maplist) == 1)
@@ -480,6 +512,8 @@ function weathermap_fullview($cycle=FALSE, $firstonly=FALSE)
 <?php
 	html_graph_end_box();
 
+	weathermap_tabs($limit_to_group);
+	
 	$i = 0;
 	if (sizeof($maplist) > 0)
 	{
@@ -492,23 +526,24 @@ function weathermap_fullview($cycle=FALSE, $firstonly=FALSE)
 			$htmlfile = $outdir.$map['filehash'].".html";
 			$maptitle = $map['titlecache'];
 			if($maptitle == '') $maptitle= "Map for config file: ".$map['configfile'];
-
+			
 			print '<div class="weathermapholder" id="mapholder_'.$map['filehash'].'">';
 			html_graph_start_box(1,true);
+			print '<tr bgcolor="#' . $colors["header_panel"] . '">'; 
 ?>
-		<tr bgcolor="#<?php print $colors["header_panel"];?>">
 				<td colspan="3">
 						<table width="100%" cellspacing="0" cellpadding="3" border="0">
 								<tr>
-										<td align="left" class="textHeaderDark"><a name="map_<?php echo $map['filehash']; ?>"></a>
-									 <?php print htmlspecialchars($maptitle); ?>
-				</td>
+									<td align="left" class="textHeaderDark">
+                                                                            <a name="map_<?php echo $map['filehash']; ?>"></a>
+                                                                            <?php print htmlspecialchars($maptitle); ?>
+                                                                        </td>
 								</tr>
 						</table>
 				</td>
 		</tr>
-	<tr>
-		<td>
+		<tr>
+			<td>
 <?php
 
 			if(file_exists($htmlfile))
@@ -706,6 +741,135 @@ function weathermap_footer_links()
 	html_end_box(); 
 }
 
+function weathermap_mapselector($current_id = 0)
+{
+	global $colors;
+
+    $show_selector = intval(read_config_option("weathermap_map_selector"));
+
+	if($show_selector == 0) return false;
+
+	$userid = (isset($_SESSION["sess_user_id"]) ? intval($_SESSION["sess_user_id"]) : 1);
+	$maps = db_fetch_assoc("select distinct weathermap_maps.*,weathermap_groups.name, weathermap_groups.sortorder as gsort from weathermap_groups,weathermap_auth,weathermap_maps where weathermap_maps.group_id=weathermap_groups.id and weathermap_maps.id=weathermap_auth.mapid and active='on' and (userid=".$userid." or userid=0) order by gsort, sortorder");
+
+	if(sizeof($maps)>1)
+	{
+
+		/* include graph view filter selector */
+		html_graph_start_box(3, TRUE);
+		?>
+	<tr bgcolor="<?php print $colors["panel"];?>" class="noprint">
+			<form name="weathermap_select" method="post" action="">
+			<input name="action" value="viewmap" type="hidden">
+			<td class="noprint">
+					<table width="100%" cellpadding="0" cellspacing="0">
+							<tr class="noprint">
+									<td nowrap style='white-space: nowrap;' width="40">
+										&nbsp;<strong>Jump To Map:</strong>&nbsp;
+									</td>
+									<td>
+										<select name="id">
+<?php
+
+		$ngroups = 0;
+		$lastgroup = "------lasdjflkjsdlfkjlksdjflksjdflkjsldjlkjsd";
+		foreach ($maps as $map)
+		{
+			if($current_id == $map['id']) $nullhash = $map['filehash'];
+			if($map['name'] != $lastgroup)
+			{
+				$ngroups++;
+				$lastgroup = $map['name'];
+			}
+		}
+
+
+		$lastgroup = "------lasdjflkjsdlfkjlksdjflksjdflkjsldjlkjsd";
+		foreach ($maps as $map)
+		{
+			if($ngroups>1 && $map['name'] != $lastgroup)
+			{
+				print "<option style='font-weight: bold; font-style: italic' value='$nullhash'>".htmlspecialchars($map['name'])."</option>";
+				$lastgroup = $map['name'];
+			}
+			print '<option ';
+			if($current_id == $map['id']) print " SELECTED ";
+			print 'value="'.$map['filehash'].'">';
+			// if we're showing group headings, then indent the map names
+			if($ngroups>1) { print " - "; }
+			print htmlspecialchars($map['titlecache']).'</option>';
+		}
+?>
+										</select>
+											&nbsp;<input type="image" src="../../images/button_go.gif" alt="Go" border="0" align="absmiddle">										
+									</td>
+							</tr>
+					</table>
+			</td>
+			</form>
+	</tr>
+	<?php
+
+		html_graph_end_box(FALSE);
+	}
+}
+
+function weathermap_get_valid_tabs()
+{
+	$tabs = array();
+	$userid = (isset($_SESSION["sess_user_id"]) ? intval($_SESSION["sess_user_id"]) : 1);
+	$maps = db_fetch_assoc("select weathermap_maps.*, weathermap_groups.name as group_name from weathermap_auth,weathermap_maps, weathermap_groups where weathermap_groups.id=weathermap_maps.group_id and weathermap_maps.id=weathermap_auth.mapid and active='on' and (userid=".$userid." or userid=0) order by weathermap_groups.sortorder");
+
+
+	foreach ($maps as $map)
+	{
+		$tabs[$map['group_id']] = $map['group_name'];
+	}
+
+	return($tabs);
+}
+
+function weathermap_tabs($current_tab)
+{
+	global $colors;
+
+	// $current_tab=2;
+
+	$tabs = weathermap_get_valid_tabs();
+	
+	# print "Limiting to $current_tab\n";
+	
+	if(sizeof($tabs) > 1)
+	{
+		/* draw the categories tabs on the top of the page */
+        print "<p></p><table class='tabs' width='100%' cellspacing='0' cellpadding='3' align='center'><tr>\n";
+
+        if (sizeof($tabs) > 0) {
+		$show_all = intval(read_config_option("weathermap_all_tab"));
+		if($show_all == 1)
+		{
+			$tabs['-2'] = "All Maps";
+		}
+
+	        foreach (array_keys($tabs) as $tab_short_name) {
+	                print "<td " . (($tab_short_name == $current_tab) ? "bgcolor='silver'" : "bgcolor='#DFDFDF'") . " nowrap='nowrap' width='" . (strlen($tabs[$tab_short_name]) * 9) . "' align='center' class='tab'>
+	                                <span class='textHeader'><a href='weathermap-cacti-plugin.php?group_id=$tab_short_name'>$tabs[$tab_short_name]</a></span>
+	                                </td>\n
+	                                <td width='1'></td>\n";
+	        }
+
+        }
+
+        print "<td></td>\n</tr></table>\n";
+		
+		return(true);
+	}
+	else
+	{
+		return(false);
+	}		
+		
+}
 
 // vim:ts=4:sw=4:
 ?>
